package org.sample.nio.command;

import org.sample.nio.QuoteManager;
import org.sample.nio.connection.handler.ClientConnectionHandler;

/**
 * The quote Command must be in the format
 * 
 * quit|symbol[eoq]
 * 
 * The client want to stop receiving update
 * for the feed.
 * 
 * if there is no more client subscribe to the
 * quotesubscription, the feed will be close with
 * the 3th party
 * 
 *  
 * @author Sebastien Dionne
 *
 */
public class QuitCommand extends BasicCommand { 
	
	public QuitCommand(QuoteManager manager) {
		super(manager);
	}

	/**
	 * process the query
	 * @param query query to send to 3th party
	 * @param clientConnectionHandler client that call the command
	 * @return BFTransactionReq the request
	 */
	public ICommandRequest process(String query, ClientConnectionHandler clientConnectionHandler){
		
		// we need to close the feed that the client was subcribed
		QuoteManager manager = getQuoteManager();
		
		manager.unsubcribeClient(clientConnectionHandler);
		
		clientConnectionHandler.close();
		
		return null;
	}

}
