package org.sample.nio.command;

import org.sample.nio.QuoteManager;
import org.sample.nio.connection.handler.ClientConnectionHandler;
import org.sample.nio.connection.handler.ThirdPartyConnectionHandler;
import org.sample.nio.response.handler.IResponseHandler;
import org.sample.nio.response.handler.QuoteResponseHandler;

/**
 * The quote Command must be in the format
 * 
 * quote|symbol[eoq]
 * 
 * The server will return the latest quote from
 * the cache, if there is a feed subscribe for this
 * symbol.  If not, the server will ask the quote at
 * the 3th party, and when the response is received,
 * it will be send to be client.
 * 
 * Once the client received the response, the client 
 * connection will be closed.
 * 
 * @author Sebastien Dionne
 *
 */
public class QuoteCommand extends BasicCommand {
	
	public QuoteCommand(QuoteManager manager) {
		super(manager);
	}

	/**
	 * process the query
	 * @param query query to send to 3th party
	 * @param clientConnectionHandler client that call the command
	 * @return CommandRequest the request 
	 */
	public ICommandRequest process(String query, ClientConnectionHandler clientConnectionHandler){
		
		// extract the quote from the query
		int index = query.indexOf(QuoteManager.ARG_SEP);
		if(index<=0){
			//error
			return null;
		}
		
		f_quoteSubscription = query.substring(index+QuoteManager.ARG_SEP.length());
		
		f_responseHandler = new QuoteResponseHandler();
		f_responseHandler.setClientConnectionHandler(clientConnectionHandler);
		f_responseHandler.setCommand(this);
		
		// look if the quoteSubscription is in the cache
		IResponseHandler handler = f_manager.getResponseHandlerFromQuoteSubcription(f_quoteSubscription);
		if(handler!=null){
			System.out.println("QUOTE FOUND IN CACHE = " + f_quoteSubscription);
			
			handler.sendLastUpdateToClient(clientConnectionHandler);
			return handler.getCommandRequest();
		} 
		
		ThirdPartyConnectionHandler connection = f_manager.getThirdPartyConnectionHandler();
		
		if (connection == null) {
			return null;
		}

		// Create a new request to the 3th party
		try {
			return connection.sendQuoteRequest(f_quoteSubscription, f_responseHandler);
		} catch (Exception e) {
			// here we could send a message to the client.. but it's a demo
			System.out.println(e);
		}
		
		return null;
	}

}
