package org.sample.nio.connection.handler;


import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.util.Iterator;

import org.sample.nio.QuoteManager;

/**
 * This class listen for incoming connection
 * 
 * @author Sebastien Dionne
 *
 */
public class SocketConnectionListener implements Runnable {
	
	protected String f_name;
	protected int f_port;
	protected QuoteManager f_quoteManager;
	protected boolean f_shutdown;
	protected ThreadGroup f_threadGroup;
	protected ServerSocketChannel f_serverSocket;
	protected ServerSocketChannel f_serverSocketChannel;
	protected Selector f_selector;
	
	/**
	 * @return the quoteManager
	 */
	public QuoteManager getQuoteManager() {
		return f_quoteManager;
	}

	/**
	 * @param quoteManager the quoteManager
	 */
	public void setQuoteManager(QuoteManager quoteManager) {
		this.f_quoteManager = quoteManager;
	}
	
	/**
	 * 
	 * @param port port for the incomming connection
	 */
	public void setPort(int port){
		f_port = port;
	}
	
	/**
	 * 
	 * @return the listening port
	 */
	public int getPort(){
		return f_port;
	}

	/**
	 * Init
	 */
	public void init(){
		
		System.out.println("listening for incomming TCP Connections on port : " + f_port);
		try {
			// Create a non-blocking server socket channel and bind to the port
			f_serverSocketChannel = ServerSocketChannel.open();
			f_serverSocketChannel.configureBlocking(false);
			f_serverSocketChannel.socket().bind(new InetSocketAddress(f_port));
			
			// Create the selector and bind the server socket to it
            f_selector = Selector.open();
            f_serverSocketChannel.register(f_selector, SelectionKey.OP_ACCEPT, new ConnectionAcceptor(f_selector, f_serverSocketChannel, f_quoteManager));
            
			f_threadGroup = new ThreadGroup("ClientConnectionHandlerGroup");
			
		} catch (Exception e) {
			System.exit(-10);
		}
	}

	/**
	 * @return the shutdown
	 */
	public boolean isShutdown() {
		return f_shutdown;
	}

	/**
	 * shutdown the socket listener
	 */
	public void shutdown() {
		f_shutdown = true;
		// A TESTER
		f_threadGroup.interrupt();
	}
	
	/**
	 * processing
	 * 
	 * - Open socket
	 * - a new Thread (ClientConnectionHandler)
	 * - start the new thread
	 */
	public void run() {

		init();
		
		while(!isShutdown()){
			try {
				
				f_selector.select();

                // Get list of selection keys with pending events
                Iterator<SelectionKey> it = f_selector.selectedKeys().iterator();

                // Process each key
                while (it.hasNext()) {
                    // Get the selection key
                    SelectionKey selKey = it.next();

                    // Remove it from the list to indicate that it is being processed
                    it.remove();

                    // Check if it's a connection request
                    if (selKey.isValid() && selKey.isAcceptable()) {
                    	ConnectionAcceptor connectionAcceptor = (ConnectionAcceptor)selKey.attachment();
                        connectionAcceptor.accept();
                    }
                    
                    // Check if a message has been sent
                    if (selKey.isValid() && selKey.isReadable()) {
                    	ClientConnectionHandler connectionReader = (ClientConnectionHandler)selKey.attachment();
                        connectionReader.read();
                    }
                }
				
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
	}
	
	
}
