package org.sample.nio.response;

import org.sample.nio.command.ICommandRequest;
import org.sample.nio.dao.Quote;
import org.sample.nio.response.handler.IResponseHandler;

/**
 * This class simulate a request to the 3th party
 * 
 * The quote are simulated by a Thread in the class.
 * 
 * 
 * @author Sebastien Dionne
 *
 */
public class QuoteCommandRequest implements ICommandRequest {

	public static int REQUEST_INDEX=0;
	
	private int requestID=-1;
	private IResponseHandler f_responseHandler;
	
	private String quoteSubscription;
	
	public QuoteCommandRequest(){
		REQUEST_INDEX++;
	}
	
	@Override
	public void close() {
		//only a demo.. 
	}

	@Override
	public String getQuoteSubscription() {
		return quoteSubscription;
	}
	
	public void setQuoteSubscription(String quoteSubscription) {
		this.quoteSubscription = quoteSubscription;
	}

	@Override
	public int getRequestID() {
		return requestID;
	}

	@Override
	public IResponseHandler getResponseHandler() {
		return f_responseHandler;
	}

	public void setResponseHandler(IResponseHandler responseHandler){
		f_responseHandler = responseHandler;
	}
	
	
	public void startGenerateQuote(){
		
		Thread t = new Thread(new Runnable(){ public void run(){
			
			try {
				double bid = 50 * Math.random() + 1;
				double ask = 50 * Math.random() + 1;
				
				// send the generated quote to client
				handleQuote(new Quote(getQuoteSubscription(),bid,ask));
				
			} catch(Exception e){
				e.printStackTrace();
			}
		}
			
		});
		
		// wait 1 sec.. simulate a little delay
		try {
			Thread.sleep(1000);
		} catch(Exception e){
			e.printStackTrace();
		}
		
		t.start();
		
	}
	
	/**
	 * handle the quote receive by the 3th party and send 
	 * this quote to the client(s)
	 *  @param quote the quote received by the 3th party
	 */
	public void handleQuote(Quote quote){
		// send the generated quote to client
		StringBuffer sb = new StringBuffer(quote.toString()).append("\n");
		f_responseHandler.sendToClient(sb);
		
		// close the connection because the client requested only a quote
		f_responseHandler.getClientConnectionHandler().close();
		
	}
}
